@@ -1,5 +1,6 @@
 //! Command processor
 
+use log::debug;
 use std::cell::RefCell;
 use std::fmt;
 use std::rc::Rc;
@@ -10,10 +11,12 @@ use super::{Context, Helper, Result};
 use crate::highlight::Highlighter;
 use crate::history::Direction;
 use crate::keymap::{Anchor, At, CharSearch, Cmd, Movement, RepeatCount, Word};
-use crate::keymap::{InputState, Refresher};
+use crate::keymap::{InputState, Invoke, Refresher};
+use crate::layout::{Layout, Position};
 use crate::line_buffer::{LineBuffer, WordAction, MAX_LINE};
-use crate::tty::{Position, Renderer, Term, Terminal};
+use crate::tty::{Renderer, Term, Terminal};
 use crate::undo::Changeset;
+use crate::validate::{ValidationContext, ValidationResult};
 
 /// Represent the state during line editing.
 /// Implement rendering.
@@ -22,9 +25,7 @@ pub struct State<'out, 'prompt, H: Helper> {
     prompt: &'prompt str,  // Prompt to display (rl_prompt)
     prompt_size: Position, // Prompt Unicode/visible width and height
     pub line: LineBuffer,  // Edited line buffer
-    pub cursor: Position,  /* Cursor position (relative to the start of the prompt
-                            * for `row`) */
-    pub old_rows: usize, // Number of rows used so far (from start of prompt to end of input)
+    pub layout: Layout,
     saved_line_for_history: LineBuffer, // Current edited line before history browsing
     byte_buffer: [u8; 4],
     pub changes: Rc<RefCell<Changeset>>, // changes to line, for undo/redo
@@ -47,16 +48,14 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         helper: Option<&'out H>,
         ctx: Context<'out>,
     ) -> State<'out, 'prompt, H> {
-        let capacity = MAX_LINE;
         let prompt_size = out.calculate_position(prompt, Position::default());
         State {
             out,
             prompt,
             prompt_size,
-            line: LineBuffer::with_capacity(capacity),
-            cursor: prompt_size,
-            old_rows: 0,
-            saved_line_for_history: LineBuffer::with_capacity(capacity),
+            line: LineBuffer::with_capacity(MAX_LINE).can_growth(true),
+            layout: Layout::default(),
+            saved_line_for_history: LineBuffer::with_capacity(MAX_LINE).can_growth(true),
             byte_buffer: [0; 4],
             changes: Rc::new(RefCell::new(Changeset::new())),
             helper,
@@ -84,6 +83,9 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             let rc = input_state.next_cmd(rdr, self, single_esc_abort);
             if rc.is_err() && self.out.sigwinch() {
                 self.out.update_size();
+                self.prompt_size = self
+                    .out
+                    .calculate_position(self.prompt, Position::default());
                 self.refresh_line()?;
                 continue;
             }
@@ -111,16 +113,19 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         let cursor = self
             .out
             .calculate_position(&self.line[..self.line.pos()], self.prompt_size);
-        if self.cursor == cursor {
+        if self.layout.cursor == cursor {
             return Ok(());
         }
         if self.highlight_char() {
             let prompt_size = self.prompt_size;
             self.refresh(self.prompt, prompt_size, true, Info::NoHint)?;
         } else {
-            self.out.move_cursor(self.cursor, cursor)?;
+            self.out.move_cursor(self.layout.cursor, cursor)?;
+            self.layout.prompt_size = self.prompt_size;
+            self.layout.cursor = cursor;
+            debug_assert!(self.layout.prompt_size <= self.layout.cursor);
+            debug_assert!(self.layout.cursor <= self.layout.end);
         }
-        self.cursor = cursor;
         Ok(())
     }
 
@@ -145,19 +150,41 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         } else {
             None
         };
-        let (cursor, end_pos) = self.out.refresh_line(
-            prompt,
+
+        // calculate the desired position of the cursor
+        let pos = self.line.pos();
+        let cursor = self.out.calculate_position(&self.line[..pos], prompt_size);
+        // calculate the position of the end of the input line
+        let mut end = if pos == self.line.len() {
+            cursor
+        } else {
+            self.out.calculate_position(&self.line[pos..], cursor)
+        };
+        if let Some(info) = info {
+            end = self.out.calculate_position(&info, end);
+        }
+
+        let new_layout = Layout {
             prompt_size,
             default_prompt,
+            cursor,
+            end,
+        };
+        debug_assert!(new_layout.prompt_size <= new_layout.cursor);
+        debug_assert!(new_layout.cursor <= new_layout.end);
+
+        debug!(target: "rustyline", "old layout: {:?}", self.layout);
+        debug!(target: "rustyline", "new layout: {:?}", new_layout);
+        self.out.refresh_line(
+            prompt,
             &self.line,
             info,
-            self.cursor.row,
-            self.old_rows,
+            &self.layout,
+            &new_layout,
             highlighter,
         )?;
+        self.layout = new_layout;
 
-        self.cursor = cursor;
-        self.old_rows = end_pos.row;
         Ok(())
     }
 
@@ -187,6 +214,50 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
             false
         }
     }
+
+    pub fn is_default_prompt(&self) -> bool {
+        self.layout.default_prompt
+    }
+
+    pub fn validate(&mut self) -> Result<bool> {
+        if let Some(validator) = self.helper {
+            self.changes.borrow_mut().begin();
+            let result = validator.validate(&mut ValidationContext::new(self))?;
+            let corrected = self.changes.borrow_mut().end();
+            let validated = match result {
+                ValidationResult::Incomplete => {
+                    self.edit_move_end()?;
+                    self.edit_insert('\n', 1)?;
+                    false
+                }
+                ValidationResult::Valid(msg) => {
+                    // Accept the line regardless of where the cursor is.
+                    self.edit_move_end()?;
+                    if corrected || self.has_hint() || msg.is_some() {
+                        // Force a refresh without hints to leave the previous
+                        // line as the user typed it after a newline.
+                        self.refresh_line_with_msg(msg)?;
+                    }
+                    true
+                }
+                ValidationResult::Invalid(msg) => {
+                    if corrected || self.has_hint() || msg.is_some() {
+                        self.refresh_line_with_msg(msg)?;
+                    }
+                    false
+                }
+            };
+            Ok(validated)
+        } else {
+            Ok(true)
+        }
+    }
+}
+
+impl<'out, 'prompt, H: Helper> Invoke for State<'out, 'prompt, H> {
+    fn input(&self) -> &str {
+        self.line.as_str()
+    }
 }
 
 impl<'out, 'prompt, H: Helper> Refresher for State<'out, 'prompt, H> {
@@ -243,15 +314,21 @@ impl<'out, 'prompt, H: Helper> fmt::Debug for State<'out, 'prompt, H> {
             .field("prompt", &self.prompt)
             .field("prompt_size", &self.prompt_size)
             .field("buf", &self.line)
-            .field("cursor", &self.cursor)
             .field("cols", &self.out.get_columns())
-            .field("old_rows", &self.old_rows)
+            .field("layout", &self.layout)
             .field("saved_line_for_history", &self.saved_line_for_history)
             .finish()
     }
 }
 
 impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
+    pub fn clear_screen(&mut self) -> Result<()> {
+        self.out.clear_screen()?;
+        self.layout.cursor = Position::default();
+        self.layout.end = Position::default();
+        Ok(())
+    }
+
     /// Insert the character `ch` at cursor current position.
     pub fn edit_insert(&mut self, ch: char, n: RepeatCount) -> Result<()> {
         if let Some(push) = self.line.insert(ch, n) {
@@ -259,16 +336,18 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
                 let prompt_size = self.prompt_size;
                 let no_previous_hint = self.hint.is_none();
                 self.hint();
+                let width = ch.width().unwrap_or(0);
                 if n == 1
-                    && self.cursor.col + ch.width().unwrap_or(0) < self.out.get_columns()
+                    && width != 0 // Ctrl-V + \t or \n ...
+                    && self.layout.cursor.col + width < self.out.get_columns()
                     && (self.hint.is_none() && no_previous_hint) // TODO refresh only current line
                     && !self.highlight_char()
                 {
                     // Avoid a full update of the line in the trivial case.
-                    let cursor = self
-                        .out
-                        .calculate_position(&self.line[..self.line.pos()], self.prompt_size);
-                    self.cursor = cursor;
+                    self.layout.cursor.col += width;
+                    self.layout.end.col += width;
+                    debug_assert!(self.layout.prompt_size <= self.layout.cursor);
+                    debug_assert!(self.layout.cursor <= self.layout.end);
                     let bits = ch.encode_utf8(&mut self.byte_buffer);
                     let bits = bits.as_bytes();
                     self.out.write_and_flush(bits)
@@ -578,16 +657,12 @@ pub fn init_state<'out, H: Helper>(
         prompt: "",
         prompt_size: Position::default(),
         line: LineBuffer::init(line, pos, None),
-        cursor: Position::default(),
-        old_rows: 0,
+        layout: Layout::default(),
         saved_line_for_history: LineBuffer::with_capacity(100),
         byte_buffer: [0; 4],
         changes: Rc::new(RefCell::new(Changeset::new())),
         helper,
-        ctx: Context {
-            history,
-            history_index: 0,
-        },
+        ctx: Context::new(history),
         hint: Some("hint".to_owned()),
         highlight_char: false,
     }
