@@ -2,6 +2,8 @@
 use std::collections::HashMap;
 use std::sync::{Arc, RwLock};
 
+use log::debug;
+
 use super::Result;
 use crate::config::Config;
 use crate::config::EditMode;
@@ -27,6 +29,8 @@ pub enum Cmd {
     ClearScreen,
     /// complete
     Complete,
+    /// complete-backward
+    CompleteBackward,
     /// complete-hint
     CompleteHint,
     /// downcase-word
@@ -284,6 +288,14 @@ pub struct InputState {
     last_char_search: Option<CharSearch>, // vi only
 }
 
+/// Provide indirect mutation to user input.
+pub trait Invoke {
+    /// currently edited line
+    fn input(&self) -> &str;
+    // TODO
+    //fn invoke(&mut self, cmd: Cmd) -> Result<?>;
+}
+
 pub trait Refresher {
     /// Rewrite the currently edited line accordingly to the buffer content,
     /// cursor position, and number of columns of the terminal.
@@ -336,7 +348,6 @@ impl InputState {
         }
     }
 
-    // TODO dynamic prompt (arg: ?)
     fn emacs_digit_argument<R: RawReader>(
         &mut self,
         rdr: &mut R,
@@ -434,7 +445,14 @@ impl InputState {
                     Cmd::Kill(Movement::ForwardChar(n))
                 }
             }
-            KeyPress::Tab => Cmd::Complete,
+            KeyPress::BackTab => Cmd::CompleteBackward,
+            KeyPress::Tab => {
+                if positive {
+                    Cmd::Complete
+                } else {
+                    Cmd::CompleteBackward
+                }
+            }
             // Don't complete hints when the cursor is not at the end of a line
             KeyPress::Right if wrt.has_hint() && wrt.is_cursor_at_end() => Cmd::CompleteHint,
             KeyPress::Ctrl('K') => {
@@ -717,6 +735,7 @@ impl InputState {
                 }
             }
             KeyPress::Ctrl('H') | KeyPress::Backspace => Cmd::Kill(Movement::BackwardChar(1)),
+            KeyPress::BackTab => Cmd::CompleteBackward,
             KeyPress::Tab => Cmd::Complete,
             // Don't complete hints when the cursor is not at the end of a line
             KeyPress::Right if wrt.has_hint() && wrt.is_cursor_at_end() => Cmd::CompleteHint,
