@@ -17,12 +17,6 @@
 //! ```
 // #![feature(non_exhaustive)]
 
-#[macro_use]
-extern crate log;
-
-#[cfg(unix)]
-extern crate nix;
-
 pub mod completion;
 pub mod config;
 mod edit;
@@ -33,10 +27,11 @@ pub mod history;
 mod keymap;
 mod keys;
 mod kill_ring;
+mod layout;
 pub mod line_buffer;
-mod undo;
-
 mod tty;
+mod undo;
+pub mod validate;
 
 use std::collections::HashMap;
 use std::fmt;
@@ -44,9 +39,11 @@ use std::io::{self, Write};
 use std::path::Path;
 use std::result;
 use std::sync::{Arc, Mutex, RwLock};
+
+use log::debug;
 use unicode_width::UnicodeWidthStr;
 
-use crate::tty::{RawMode, RawReader, Renderer, Term, Terminal};
+use crate::tty::{RawMode, Renderer, Term, Terminal};
 
 use crate::completion::{longest_common_prefix, Candidate, Completer};
 pub use crate::config::{
@@ -61,6 +58,7 @@ use crate::keymap::{InputState, Refresher};
 pub use crate::keys::KeyPress;
 use crate::kill_ring::{KillRing, Mode};
 use crate::line_buffer::WordAction;
+use crate::validate::Validator;
 
 /// The error type for I/O and Linux Syscalls (Errno)
 pub type Result<T> = result::Result<T, error::ReadlineError>;
@@ -112,6 +110,14 @@ fn complete_line<H: Helper>(
                         s.out.beep()?;
                     }
                 }
+                Cmd::CompleteBackward => {
+                    if i == 0 {
+                        i = candidates.len(); // Circular
+                        s.out.beep()?;
+                    } else {
+                        i = (i - 1) % (candidates.len() + 1); // Circular
+                    }
+                }
                 Cmd::Abort => {
                     // Re-show original buffer
                     if i < candidates.len() {
@@ -156,7 +162,7 @@ fn complete_line<H: Helper>(
         let show_completions = if candidates.len() > config.completion_prompt_limit() {
             let msg = format!("\nDisplay all {} possibilities? (y or n)", candidates.len());
             s.out.write_and_flush(msg.as_bytes())?;
-            s.old_rows += 1;
+            s.layout.end.row += 1;
             while cmd != Cmd::SelfInsert(1, 'y')
                 && cmd != Cmd::SelfInsert(1, 'Y')
                 && cmd != Cmd::SelfInsert(1, 'n')
@@ -375,10 +381,7 @@ fn readline_edit<H: Helper>(
     let mut stdout = editor.term.create_writer();
 
     editor.reset_kill_ring(); // TODO recreate a new kill ring vs Arc<Mutex<KillRing>>
-    let ctx = Context {
-        history: &editor.history,
-        history_index: editor.history.len(),
-    };
+    let ctx = Context::new(&editor.history);
     let mut s = State::new(&mut stdout, prompt, helper, ctx);
     let mut input_state = InputState::new(&editor.config, Arc::clone(&editor.custom_bindings));
 
@@ -391,7 +394,9 @@ fn readline_edit<H: Helper>(
     }
 
     let mut rdr = editor.term.create_reader(&editor.config)?;
-    s.move_cursor_at_leftmost(&mut rdr)?;
+    if editor.term.is_output_tty() {
+        s.move_cursor_at_leftmost(&mut rdr)?;
+    }
     s.refresh_line()?;
 
     loop {
@@ -479,7 +484,7 @@ fn readline_edit<H: Helper>(
             }
             Cmd::ClearScreen => {
                 // Clear the screen leaving the current line at the top of the screen.
-                s.out.clear_screen()?;
+                s.clear_screen()?;
                 s.refresh_line()?
             }
             Cmd::NextHistory => {
@@ -499,6 +504,7 @@ fn readline_edit<H: Helper>(
             #[cfg(unix)]
             Cmd::QuotedInsert => {
                 // Quoted insert
+                use tty::RawReader;
                 let c = rdr.next_char()?;
                 s.edit_insert(c, 1)?
             }
@@ -518,16 +524,19 @@ fn readline_edit<H: Helper>(
             Cmd::AcceptLine => {
                 #[cfg(test)]
                 {
-                    editor.term.cursor = s.cursor.col;
+                    editor.term.cursor = s.layout.cursor.col;
                 }
                 // Accept the line regardless of where the cursor is.
-                s.edit_move_end()?;
-                if s.has_hint() {
+                if s.has_hint() || !s.is_default_prompt() {
                     // Force a refresh without hints to leave the previous
                     // line as the user typed it after a newline.
                     s.refresh_line_with_msg(None)?;
                 }
-                break;
+                s.edit_move_end()?;
+                if s.validate()? {
+                    break;
+                }
+                continue;
             }
             Cmd::BeginningOfHistory => {
                 // move to first entry in history
@@ -647,9 +656,7 @@ fn readline_direct() -> Result<String> {
 /// (parse current line once)
 pub trait Helper
 where
-    Self: Completer,
-    Self: Hinter,
-    Self: Highlighter,
+    Self: Completer + Hinter + Highlighter + Validator,
 {
 }
 
@@ -664,6 +671,14 @@ pub struct Context<'h> {
 }
 
 impl<'h> Context<'h> {
+    /// Constructor. Visible for testing.
+    pub fn new(history: &'h History) -> Self {
+        Context {
+            history,
+            history_index: history.len(),
+        }
+    }
+
     /// Return an immutable reference to the history object.
     pub fn history(&self) -> &History {
         &self.history
@@ -698,6 +713,7 @@ impl<H: Helper> Editor<H> {
             config.color_mode(),
             config.output_stream(),
             config.tab_stop(),
+            config.bell_style(),
         );
         Self {
             term,
