@@ -9,8 +9,8 @@ use std::string::Drain;
 use std::sync::{Arc, Mutex};
 use unicode_segmentation::UnicodeSegmentation;
 
-/// Maximum buffer size for the line read
-pub(crate) static MAX_LINE: usize = 4096;
+/// Default maximum buffer size for the line read
+pub(crate) const MAX_LINE: usize = 4096;
 
 /// Word's case change
 #[derive(Clone, Copy)]
@@ -51,8 +51,9 @@ pub(crate) trait ChangeListener: DeleteListener {
 ///
 /// The methods do text manipulations or/and cursor movements.
 pub struct LineBuffer {
-    buf: String, // Edited line buffer (rl_line_buffer)
-    pos: usize,  // Current cursor position (byte position) (rl_point)
+    buf: String,      // Edited line buffer (rl_line_buffer)
+    pos: usize,       // Current cursor position (byte position) (rl_point)
+    can_growth: bool, // Whether to allow dynamic growth
     dl: Option<Arc<Mutex<dyn DeleteListener>>>,
     cl: Option<Rc<RefCell<dyn ChangeListener>>>,
 }
@@ -72,11 +73,22 @@ impl LineBuffer {
         Self {
             buf: String::with_capacity(capacity),
             pos: 0,
+            can_growth: false,
             dl: None,
             cl: None,
         }
     }
 
+    /// Set whether to allow dynamic allocation
+    pub(crate) fn can_growth(mut self, can_growth: bool) -> Self {
+        self.can_growth = can_growth;
+        self
+    }
+
+    fn must_truncate(&self, new_len: usize) -> bool {
+        !self.can_growth && new_len > self.buf.capacity()
+    }
+
     #[cfg(test)]
     pub(crate) fn init(
         line: &str,
@@ -135,7 +147,7 @@ impl LineBuffer {
         let end = self.len();
         self.drain(0..end, Direction::default());
         let max = self.buf.capacity();
-        if buf.len() > max {
+        if self.must_truncate(buf.len()) {
             self.insert_str(0, &buf[..max]);
             if pos > max {
                 self.pos = max;
@@ -190,7 +202,7 @@ impl LineBuffer {
     /// `true` when the character has been appended to the end of the line.
     pub fn insert(&mut self, ch: char, n: RepeatCount) -> Option<bool> {
         let shift = ch.len_utf8() * n;
-        if self.buf.len() + shift > self.buf.capacity() {
+        if self.must_truncate(self.buf.len() + shift) {
             return None;
         }
         let push = self.pos == self.buf.len();
@@ -199,8 +211,8 @@ impl LineBuffer {
             for cl in &self.cl {
                 if let Ok(mut cl) = cl.try_borrow_mut() {
                     cl.insert_char(self.pos, ch);
-                } // Ok: while undoing, cl is borrowed. And we want to ignore changes while
-                  // undoing.
+                } // Ok: while undoing, cl is borrowed. And we want to ignore
+                  // changes while undoing.
             }
         } else {
             let text = iter::repeat(ch).take(n).collect::<String>();
@@ -212,11 +224,11 @@ impl LineBuffer {
     }
 
     /// Yank/paste `text` at current position.
-    /// Return `None` when maximum buffer size has been reached,
+    /// Return `None` when maximum buffer size has been reached or is empty,
     /// `true` when the character has been appended to the end of the line.
     pub fn yank(&mut self, text: &str, n: RepeatCount) -> Option<bool> {
         let shift = text.len() * n;
-        if text.is_empty() || (self.buf.len() + shift) > self.buf.capacity() {
+        if text.is_empty() || self.must_truncate(self.buf.len() + shift) {
             return None;
         }
         let push = self.pos == self.buf.len();
@@ -645,8 +657,8 @@ impl LineBuffer {
         for cl in &self.cl {
             if let Ok(mut cl) = cl.try_borrow_mut() {
                 cl.replace(start, self.buf.index(range.clone()), text);
-            } // Ok: while undoing, cl is borrowed. And we want to ignore changes while
-              // undoing.
+            } // Ok: while undoing, cl is borrowed. And we want to ignore
+              // changes while undoing.
         }
         self.buf.drain(range);
         if start == self.buf.len() {
@@ -663,8 +675,8 @@ impl LineBuffer {
         for cl in &self.cl {
             if let Ok(mut cl) = cl.try_borrow_mut() {
                 cl.insert_str(idx, s);
-            } // Ok: while undoing, cl is borrowed. And we want to ignore changes while
-              // undoing.
+            } // Ok: while undoing, cl is borrowed. And we want to ignore
+              // changes while undoing.
         }
         if idx == self.buf.len() {
             self.buf.push_str(s);
@@ -691,8 +703,8 @@ impl LineBuffer {
         for cl in &self.cl {
             if let Ok(mut cl) = cl.try_borrow_mut() {
                 cl.delete(range.start, &self.buf[range.start..range.end], dir);
-            } // Ok: while undoing, cl is borrowed. And we want to ignore changes while
-              // undoing.
+            } // Ok: while undoing, cl is borrowed. And we want to ignore
+              // changes while undoing.
         }
         self.buf.drain(range)
     }
