@@ -3,17 +3,18 @@ use std::io::{self, Write};
 use std::mem;
 use std::sync::atomic;
 
+use log::debug;
 use unicode_width::UnicodeWidthChar;
 use winapi::shared::minwindef::{DWORD, WORD};
 use winapi::um::winnt::{CHAR, HANDLE};
 use winapi::um::{consoleapi, handleapi, processenv, winbase, wincon, winuser};
 
-use super::{truncate, Position, RawMode, RawReader, Renderer, Term};
-use crate::config::OutputStreamType;
-use crate::config::{ColorMode, Config};
+use super::{RawMode, RawReader, Renderer, Term};
+use crate::config::{BellStyle, ColorMode, Config, OutputStreamType};
 use crate::error;
 use crate::highlight::Highlighter;
 use crate::keys::{self, KeyPress};
+use crate::layout::{Layout, Position};
 use crate::line_buffer::LineBuffer;
 use crate::Result;
 
@@ -125,7 +126,9 @@ impl RawReader for ConsoleRawReader {
             if rec.EventType == wincon::WINDOW_BUFFER_SIZE_EVENT {
                 SIGWINCH.store(true, atomic::Ordering::SeqCst);
                 debug!(target: "rustyline", "SIGWINCH");
-                return Err(error::ReadlineError::WindowResize); // sigwinch + err => err ignored
+                return Err(error::ReadlineError::WindowResize); // sigwinch +
+                                                                // err => err
+                                                                // ignored
             } else if rec.EventType != wincon::KEY_EVENT {
                 continue;
             }
@@ -249,10 +252,16 @@ pub struct ConsoleRenderer {
     cols: usize, // Number of columns in terminal
     buffer: String,
     colors_enabled: bool,
+    bell_style: BellStyle,
 }
 
 impl ConsoleRenderer {
-    fn new(handle: HANDLE, out: OutputStreamType, colors_enabled: bool) -> ConsoleRenderer {
+    fn new(
+        handle: HANDLE,
+        out: OutputStreamType,
+        colors_enabled: bool,
+        bell_style: BellStyle,
+    ) -> ConsoleRenderer {
         // Multi line editing is enabled by ENABLE_WRAP_AT_EOL_OUTPUT mode
         let (cols, _) = get_win_size(handle);
         ConsoleRenderer {
@@ -261,6 +270,7 @@ impl ConsoleRenderer {
             cols,
             buffer: String::with_capacity(1024),
             colors_enabled,
+            bell_style,
         }
     }
 
@@ -292,77 +302,75 @@ impl Renderer for ConsoleRenderer {
     type Reader = ConsoleRawReader;
 
     fn move_cursor(&mut self, old: Position, new: Position) -> Result<()> {
-        let mut info = self.get_console_screen_buffer_info()?;
+        let mut cursor = self.get_console_screen_buffer_info()?.dwCursorPosition;
         if new.row > old.row {
-            info.dwCursorPosition.Y += (new.row - old.row) as i16;
+            cursor.Y += (new.row - old.row) as i16;
         } else {
-            info.dwCursorPosition.Y -= (old.row - new.row) as i16;
+            cursor.Y -= (old.row - new.row) as i16;
         }
         if new.col > old.col {
-            info.dwCursorPosition.X += (new.col - old.col) as i16;
+            cursor.X += (new.col - old.col) as i16;
         } else {
-            info.dwCursorPosition.X -= (old.col - new.col) as i16;
+            cursor.X -= (old.col - new.col) as i16;
         }
-        self.set_console_cursor_position(info.dwCursorPosition)
+        self.set_console_cursor_position(cursor)
     }
 
     fn refresh_line(
         &mut self,
         prompt: &str,
-        prompt_size: Position,
-        default_prompt: bool,
         line: &LineBuffer,
         hint: Option<&str>,
-        current_row: usize,
-        old_rows: usize,
+        old_layout: &Layout,
+        new_layout: &Layout,
         highlighter: Option<&dyn Highlighter>,
-    ) -> Result<(Position, Position)> {
-        // calculate the position of the end of the input line
-        let end_pos = self.calculate_position(line, prompt_size);
-        // calculate the desired position of the cursor
-        let cursor = self.calculate_position(&line[..line.pos()], prompt_size);
+    ) -> Result<()> {
+        let default_prompt = new_layout.default_prompt;
+        let cursor = new_layout.cursor;
+        let end_pos = new_layout.end;
+        let current_row = old_layout.cursor.row;
+        let old_rows = old_layout.end.row;
 
-        // position at the start of the prompt, clear to end of previous input
-        let mut info = self.get_console_screen_buffer_info()?;
-        info.dwCursorPosition.X = 0;
-        info.dwCursorPosition.Y -= current_row as i16;
-        self.set_console_cursor_position(info.dwCursorPosition)?;
-        self.clear(
-            (info.dwSize.X * (old_rows as i16 + 1)) as DWORD,
-            info.dwCursorPosition,
-        )?;
         self.buffer.clear();
         if let Some(highlighter) = highlighter {
             // TODO handle ansi escape code (SetConsoleTextAttribute)
-            // display the prompt
+            // append the prompt
             self.buffer
                 .push_str(&highlighter.highlight_prompt(prompt, default_prompt));
-            // display the input line
+            // append the input line
             self.buffer
                 .push_str(&highlighter.highlight(line, line.pos()));
         } else {
-            // display the prompt
+            // append the prompt
             self.buffer.push_str(prompt);
-            // display the input line
+            // append the input line
             self.buffer.push_str(line);
         }
-        // display hint
+        // append hint
         if let Some(hint) = hint {
-            let truncate = truncate(&hint, end_pos.col, self.cols);
             if let Some(highlighter) = highlighter {
-                self.buffer.push_str(&highlighter.highlight_hint(truncate));
+                self.buffer.push_str(&highlighter.highlight_hint(hint));
             } else {
-                self.buffer.push_str(truncate);
+                self.buffer.push_str(hint);
             }
         }
+        // position at the start of the prompt, clear to end of previous input
+        let info = self.get_console_screen_buffer_info()?;
+        let mut coord = info.dwCursorPosition;
+        coord.X = 0;
+        coord.Y -= current_row as i16;
+        self.set_console_cursor_position(coord)?;
+        self.clear((info.dwSize.X * (old_rows as i16 + 1)) as DWORD, coord)?;
+        // display prompt, input line and hint
         self.write_and_flush(self.buffer.as_bytes())?;
 
         // position the cursor
-        let mut info = self.get_console_screen_buffer_info()?;
-        info.dwCursorPosition.X = cursor.col as i16;
-        info.dwCursorPosition.Y -= (end_pos.row - cursor.row) as i16;
-        self.set_console_cursor_position(info.dwCursorPosition)?;
-        Ok((cursor, end_pos))
+        let mut coord = self.get_console_screen_buffer_info()?.dwCursorPosition;
+        coord.X = cursor.col as i16;
+        coord.Y -= (end_pos.row - cursor.row) as i16;
+        self.set_console_cursor_position(coord)?;
+
+        Ok(())
     }
 
     fn write_and_flush(&self, buf: &[u8]) -> Result<()> {
@@ -405,6 +413,17 @@ impl Renderer for ConsoleRenderer {
         pos
     }
 
+    fn beep(&mut self) -> Result<()> {
+        match self.bell_style {
+            BellStyle::Audible => {
+                io::stderr().write_all(b"\x07")?;
+                io::stderr().flush()?;
+                Ok(())
+            }
+            _ => Ok(()),
+        }
+    }
+
     /// Clear the screen. Used to handle ctrl+l
     fn clear_screen(&mut self) -> Result<()> {
         let info = self.get_console_screen_buffer_info()?;
@@ -467,6 +486,7 @@ pub struct Console {
     pub(crate) color_mode: ColorMode,
     ansi_colors_supported: bool,
     stream_type: OutputStreamType,
+    bell_style: BellStyle,
 }
 
 impl Console {
@@ -485,7 +505,12 @@ impl Term for Console {
     type Reader = ConsoleRawReader;
     type Writer = ConsoleRenderer;
 
-    fn new(color_mode: ColorMode, stream_type: OutputStreamType, _tab_stop: usize) -> Console {
+    fn new(
+        color_mode: ColorMode,
+        stream_type: OutputStreamType,
+        _tab_stop: usize,
+        bell_style: BellStyle,
+    ) -> Console {
         use std::ptr;
         let stdin_handle = get_std_handle(STDIN_FILENO);
         let stdin_isatty = match stdin_handle {
@@ -517,6 +542,7 @@ impl Term for Console {
             color_mode,
             ansi_colors_supported: false,
             stream_type,
+            bell_style,
         }
     }
 
@@ -593,6 +619,7 @@ impl Term for Console {
             self.stdstream_handle,
             self.stream_type,
             self.colors_enabled(),
+            self.bell_style,
         )
     }
 }
